@@ -187,7 +187,8 @@ tracker_file_get_mime_type (GFile *file)
 
 gboolean
 tracker_file_system_has_enough_space (const gchar *path,
-                                      gulong       required_bytes)
+                                      gulong       required_bytes,
+                                      gboolean     creating_db)
 {
 	struct statvfs st;
 	gchar *str1;
@@ -201,25 +202,27 @@ tracker_file_system_has_enough_space (const gchar *path,
 		return FALSE;
 	}
 
-	str1 = g_format_size_for_display (required_bytes);
-	str2 = g_format_size_for_display (st.f_bsize * st.f_bavail);
-
 	enough = ((long long) st.f_bsize * st.f_bavail) >= required_bytes;
 
-	if (!enough) {
-		g_critical ("Not enough disk space to create databases, "
-		            "%s remaining, %s required as a minimum",
-		            str2,
-		            str1);
-	} else {
-		g_message ("Checking for adequate disk space to create databases, "
-		           "%s remaining, %s required as a minimum",
-		           str2,
-		           str1);
-	}
+	if (creating_db) {
+		str1 = g_format_size_for_display (required_bytes);
+		str2 = g_format_size_for_display (st.f_bsize * st.f_bavail);
 
-	g_free (str2);
-	g_free (str1);
+		if (!enough) {
+			g_critical ("Not enough disk space to create databases, "
+				    "%s remaining, %s required as a minimum",
+				    str2,
+				    str1);
+		} else {
+			g_message ("Checking for adequate disk space to create databases, "
+				   "%s remaining, %s required as a minimum",
+				   str2,
+				   str1);
+		}
+
+		g_free (str2);
+		g_free (str1);
+	}
 
 	return enough;
 }
@@ -456,17 +459,14 @@ tracker_path_list_filter_duplicates (GSList      *roots,
 			l2 = l2->next;
 		}
 
-		/* Make sure the path doesn't have the '/' suffix. */
-		p = strrchr (path, G_DIR_SEPARATOR);
+		if (G_LIKELY (!reset)) {
+			p = strrchr (path, G_DIR_SEPARATOR);
 
-		if (p) {
-			if (*(p + 1) == '\0') {
+			/* Make sure the path doesn't have the '/' suffix. */
+			if (p && !p[1]) {
 				*p = '\0';
 			}
-		}
 
-		/* Continue in list unless reset. */
-		if (G_LIKELY (!reset)) {
 			l1 = l1->next;
 		}
 	}
