@@ -58,8 +58,11 @@ struct _TrackerDataUpdateBuffer {
 	GHashTable *resource_cache;
 	/* string -> TrackerDataUpdateBufferResource */
 	GHashTable *resources;
-	/* valid per sqlite transaction, not just for same subject */
+
+	/* the following two fields are valid per sqlite transaction, not just for same subject */
 	gboolean fts_ever_updated;
+	/* TrackerClass -> integer */
+	GHashTable *class_counts;
 };
 
 struct _TrackerDataUpdateBufferResource {
@@ -72,14 +75,14 @@ struct _TrackerDataUpdateBufferResource {
 	GHashTable *predicates;
 	/* string -> TrackerDataUpdateBufferTable */
 	GHashTable *tables;
-	/* string */
+	/* TrackerClass */
 	GPtrArray *types;
 };
 
 struct _TrackerDataUpdateBufferProperty {
-	gchar *name;
+	const gchar *name;
 	GValue value;
-	gchar *graph;
+	gint graph;
 	gboolean fts : 1;
 	gboolean date_time : 1;
 };
@@ -130,6 +133,8 @@ static GPtrArray *delete_callbacks = NULL;
 static GPtrArray *commit_callbacks = NULL;
 static GPtrArray *rollback_callbacks = NULL;
 
+static gint ensure_resource_id (const gchar *uri, gboolean    *create);
+
 void
 tracker_data_add_commit_statement_callback (TrackerCommitCallback    callback,
                                             gpointer                 user_data)
@@ -276,9 +281,7 @@ cache_table_free (TrackerDataUpdateBufferTable *table)
 
 	for (i = 0; i < table->properties->len; i++) {
 		property = &g_array_index (table->properties, TrackerDataUpdateBufferProperty, i);
-		g_free (property->name);
 		g_value_unset (&property->value);
-		g_free (property->graph);
 	}
 
 	g_array_free (table->properties, TRUE);
@@ -315,7 +318,7 @@ static void
 cache_insert_value (const gchar            *table_name,
                     const gchar            *field_name,
                     GValue                 *value,
-                    const gchar            *graph,
+                    gint                    graph,
                     gboolean                multiple_values,
                     gboolean                fts,
                     gboolean                date_time)
@@ -323,9 +326,12 @@ cache_insert_value (const gchar            *table_name,
 	TrackerDataUpdateBufferTable    *table;
 	TrackerDataUpdateBufferProperty  property;
 
-	property.name = g_strdup (field_name);
+	/* No need to strdup here, the incoming string is either always static, or
+	 * long-standing as tracker_property_get_name return value. */
+	property.name = field_name;
+
 	property.value = *value;
-	property.graph = g_strdup (graph);
+	property.graph = graph;
 	property.fts = fts;
 	property.date_time = date_time;
 
@@ -356,7 +362,7 @@ cache_delete_value (const gchar            *table_name,
 
 	property.name = g_strdup (field_name);
 	property.value = *value;
-	property.graph = NULL;
+	property.graph = 0;
 	property.fts = fts;
 	property.date_time = date_time;
 
@@ -451,6 +457,24 @@ statement_bind_gvalue (TrackerDBStatement *stmt,
 	}
 }
 
+static void
+add_class_count (TrackerClass *class,
+                 gint          count)
+{
+	gint old_count_entry;
+
+	tracker_class_set_count (class, tracker_class_get_count (class) + count);
+
+	/* update class_counts table so that the count change can be reverted in case of rollback */
+	if (!update_buffer.class_counts) {
+		update_buffer.class_counts = g_hash_table_new (g_direct_hash, g_direct_equal);
+	}
+
+	old_count_entry = GPOINTER_TO_INT (g_hash_table_lookup (update_buffer.class_counts, class));
+	g_hash_table_insert (update_buffer.class_counts, class,
+	                     GINT_TO_POINTER (old_count_entry + count));
+}
+
 static void
 tracker_data_resource_buffer_flush (GError **error)
 {
@@ -517,8 +541,8 @@ tracker_data_resource_buffer_flush (GError **error)
 				tracker_db_statement_bind_int (stmt, param++, resource_buffer->id);
 				statement_bind_gvalue (stmt, &param, &property->value);
 
-				if (property->graph) {
-					tracker_db_statement_bind_int (stmt, param++, ensure_resource_id (property->graph, NULL));
+				if (property->graph != 0) {
+					tracker_db_statement_bind_int (stmt, param++, property->graph);
 				} else {
 					tracker_db_statement_bind_null (stmt, param++);
 				}
@@ -546,7 +570,7 @@ tracker_data_resource_buffer_flush (GError **error)
 				}
 
 				if (table->class) {
-					tracker_class_set_count (table->class, tracker_class_get_count (table->class) - 1);
+					add_class_count (table->class, -1);
 				}
 
 				/* remove row from class table */
@@ -631,8 +655,8 @@ tracker_data_resource_buffer_flush (GError **error)
 				} else {
 					statement_bind_gvalue (stmt, &param, &property->value);
 				}
-				if (property->graph) {
-					tracker_db_statement_bind_int (stmt, param++, ensure_resource_id (property->graph, NULL));
+				if (property->graph != 0) {
+					tracker_db_statement_bind_int (stmt, param++, property->graph);
 				} else {
 					tracker_db_statement_bind_null (stmt, param++);
 				}
@@ -685,7 +709,6 @@ static void resource_buffer_free (TrackerDataUpdateBufferResource *resource)
 	g_free (resource->subject);
 	resource->subject = NULL;
 
-	g_ptr_array_foreach (resource->types, (GFunc) g_free, NULL);
 	g_ptr_array_free (resource->types, TRUE);
 	resource->types = NULL;
 
@@ -711,6 +734,15 @@ tracker_data_update_buffer_flush (GError **error)
 	resource_buffer = NULL;
 }
 
+void
+tracker_data_update_buffer_might_flush (GError **error)
+{
+	/* avoid high memory usage by update buffer */
+	if (g_hash_table_size (update_buffer.resources) >= 1000) {
+		tracker_data_update_buffer_flush (error);
+	}
+}
+
 static void
 tracker_data_update_buffer_clear (void)
 {
@@ -719,6 +751,24 @@ tracker_data_update_buffer_clear (void)
 
 	tracker_fts_update_rollback ();
 	update_buffer.fts_ever_updated = FALSE;
+
+	if (update_buffer.class_counts) {
+		/* revert class count changes */
+
+		GHashTableIter iter;
+		TrackerClass *class;
+		gpointer count_ptr;
+
+		g_hash_table_iter_init (&iter, update_buffer.class_counts);
+		while (g_hash_table_iter_next (&iter, (gpointer*) &class, &count_ptr)) {
+			gint count;
+
+			count = GPOINTER_TO_INT (count_ptr);
+			tracker_class_set_count (class, tracker_class_get_count (class) - count);
+		}
+
+		g_hash_table_remove_all (update_buffer.class_counts);
+	}
 }
 
 static void
@@ -796,7 +846,6 @@ cache_create_service_decomposed (TrackerClass *cl,
 	TrackerClass       **super_classes;
 	GValue              gvalue = { 0 };
 	gint                i;
-	const gchar        *class_uri;
 
 	/* also create instance of all super classes */
 	super_classes = tracker_class_get_super_classes (cl);
@@ -805,27 +854,30 @@ cache_create_service_decomposed (TrackerClass *cl,
 		super_classes++;
 	}
 
-	class_uri = tracker_class_get_uri (cl);
-
 	for (i = 0; i < resource_buffer->types->len; i++) {
-		if (strcmp (g_ptr_array_index (resource_buffer->types, i), class_uri) == 0) {
+		if (g_ptr_array_index (resource_buffer->types, i) == cl) {
 			/* ignore duplicate statement */
 			return;
 		}
 	}
 
-	g_ptr_array_add (resource_buffer->types, g_strdup (class_uri));
+	g_ptr_array_add (resource_buffer->types, cl);
 
 	g_value_init (&gvalue, G_TYPE_INT);
 
 	cache_insert_row (cl);
 
 	g_value_set_int (&gvalue, ensure_resource_id (tracker_class_get_uri (cl), NULL));
-	cache_insert_value ("rdfs:Resource_rdf:type", "rdf:type", &gvalue, graph, TRUE, FALSE, FALSE);
+	cache_insert_value ("rdfs:Resource_rdf:type", "rdf:type", &gvalue,
+	                    graph != NULL ? ensure_resource_id (graph, NULL) : 0,
+	                    TRUE, FALSE, FALSE);
 
-	tracker_class_set_count (cl, tracker_class_get_count (cl) + 1);
+	add_class_count (cl, 1);
 	if (insert_callbacks) {
 		guint n;
+		const gchar *class_uri;
+
+		class_uri = tracker_class_get_uri (cl);
 
 		for (n = 0; n < insert_callbacks->len; n++) {
 			TrackerStatementDelegate *delegate;
@@ -916,8 +968,7 @@ check_property_domain (TrackerProperty *property)
 	gint type_index;
 
 	for (type_index = 0; type_index < resource_buffer->types->len; type_index++) {
-		if (strcmp (tracker_class_get_uri (tracker_property_get_domain (property)),
-		            g_ptr_array_index (resource_buffer->types, type_index)) == 0) {
+		if (tracker_property_get_domain (property) == g_ptr_array_index (resource_buffer->types, type_index)) {
 			return TRUE;
 		}
 	}
@@ -1102,7 +1153,7 @@ string_to_gvalue (const gchar         *value,
 	}
 }
 
-static void
+static gboolean
 cache_set_metadata_decomposed (TrackerProperty  *property,
                                const gchar      *value,
                                const gchar      *graph,
@@ -1115,6 +1166,7 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 	GValue              gvalue = { 0 };
 	GValueArray        *old_values;
 	GError             *new_error = NULL;
+	gboolean            change = FALSE;
 
 	/* also insert super property values */
 	super_properties = tracker_property_get_super_properties (property);
@@ -1122,7 +1174,7 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 		cache_set_metadata_decomposed (*super_properties, value, graph, &new_error);
 		if (new_error) {
 			g_propagate_error (error, new_error);
-			return;
+			return FALSE;
 		}
 		super_properties++;
 	}
@@ -1144,14 +1196,14 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 	if (new_error) {
 		g_free (table_name);
 		g_propagate_error (error, new_error);
-		return;
+		return FALSE;
 	}
 
 	string_to_gvalue (value, tracker_property_get_data_type (property), &gvalue, &new_error);
 	if (new_error) {
 		g_free (table_name);
 		g_propagate_error (error, new_error);
-		return;
+		return FALSE;
 	}
 
 	if (!value_set_add_value (old_values, &gvalue)) {
@@ -1167,14 +1219,20 @@ cache_set_metadata_decomposed (TrackerProperty  *property,
 		             resource_buffer->subject,
 		             field_name);
 	} else {
-		cache_insert_value (table_name, field_name, &gvalue, graph, multiple_values, fts,
+		cache_insert_value (table_name, field_name, &gvalue,
+		                    graph != NULL ? ensure_resource_id (graph, NULL) : 0,
+		                    multiple_values, fts,
 		                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
+
+		change = TRUE;
 	}
 
 	g_free (table_name);
+
+	return change;
 }
 
-static void
+static gboolean
 delete_metadata_decomposed (TrackerProperty  *property,
                             const gchar              *value,
                             GError          **error)
@@ -1186,6 +1244,7 @@ delete_metadata_decomposed (TrackerProperty  *property,
 	GValue gvalue = { 0 };
 	GValueArray        *old_values;
 	GError             *new_error = NULL;
+	gboolean            change = FALSE;
 
 	multiple_values = tracker_property_get_multiple_values (property);
 	if (multiple_values) {
@@ -1204,7 +1263,7 @@ delete_metadata_decomposed (TrackerProperty  *property,
 	if (new_error) {
 		g_free (table_name);
 		g_propagate_error (error, new_error);
-		return;
+		return FALSE;
 	}
 
 	string_to_gvalue (value, tracker_property_get_data_type (property), &gvalue, &new_error);
@@ -1212,7 +1271,7 @@ delete_metadata_decomposed (TrackerProperty  *property,
 	if (new_error) {
 		g_free (table_name);
 		g_propagate_error (error, new_error);
-		return;
+		return FALSE;
 	}
 
 	if (!value_set_remove_value (old_values, &gvalue)) {
@@ -1221,6 +1280,8 @@ delete_metadata_decomposed (TrackerProperty  *property,
 	} else {
 		cache_delete_value (table_name, field_name, &gvalue, multiple_values, fts,
 		                    tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME);
+
+		change = TRUE;
 	}
 
 	g_free (table_name);
@@ -1231,6 +1292,8 @@ delete_metadata_decomposed (TrackerProperty  *property,
 		delete_metadata_decomposed (*super_properties, value, error);
 		super_properties++;
 	}
+
+	return change;
 }
 
 static void
@@ -1249,7 +1312,7 @@ cache_delete_resource_type (TrackerClass *class,
 
 	found = FALSE;
 	for (i = 0; i < resource_buffer->types->len; i++) {
-		if (strcmp (g_ptr_array_index (resource_buffer->types, i), tracker_class_get_uri (class)) == 0) {
+		if (g_ptr_array_index (resource_buffer->types, i) == class) {
 			found = TRUE;
 		}
 	}
@@ -1355,6 +1418,7 @@ tracker_data_delete_statement (const gchar  *graph,
 	TrackerClass       *class;
 	TrackerProperty    *field;
 	gint                subject_id;
+	gboolean change = FALSE;
 
 	g_return_if_fail (subject != NULL);
 	g_return_if_fail (predicate != NULL);
@@ -1405,8 +1469,12 @@ tracker_data_delete_statement (const gchar  *graph,
 	} else {
 		field = tracker_ontologies_get_property_by_uri (predicate);
 		if (field != NULL) {
-			gint id = tracker_property_get_id (field);
-			if (!in_journal_replay) {
+			gint id;
+
+			change = delete_metadata_decomposed (field, object, error);
+
+			id = tracker_property_get_id (field);
+			if (!in_journal_replay && change) {
 				if (tracker_property_get_data_type (field) == TRACKER_PROPERTY_TYPE_RESOURCE) {
 					tracker_db_journal_append_delete_statement_id (
 						(graph != NULL ? query_resource_id (graph) : 0),
@@ -1421,14 +1489,12 @@ tracker_data_delete_statement (const gchar  *graph,
 						object);
 				}
 			}
-
-			delete_metadata_decomposed (field, object, error);
 		} else {
 			g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_PROPERTY,
 			             "Property '%s' not found in the ontology", predicate);
 		}
 
-		if (delete_callbacks) {
+		if (delete_callbacks && change) {
 			guint n;
 			for (n = 0; n < delete_callbacks->len; n++) {
 				TrackerStatementDelegate *delegate;
@@ -1512,7 +1578,9 @@ tracker_data_insert_statement_common (const gchar            *graph,
 		                                                 g_free, (GDestroyNotify) cache_table_free);
 
 		g_value_set_int (&gvalue, tracker_data_update_get_next_modseq ());
-		cache_insert_value ("rdfs:Resource", "tracker:modified", &gvalue, graph, FALSE, FALSE, FALSE);
+		cache_insert_value ("rdfs:Resource", "tracker:modified", &gvalue,
+		                    graph != NULL ? ensure_resource_id (graph, NULL) : 0,
+		                    FALSE, FALSE, FALSE);
 
 		g_hash_table_insert (update_buffer.resources, g_strdup (subject), resource_buffer);
 	}
@@ -1560,6 +1628,7 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 	TrackerClass    *class;
 	TrackerProperty *property;
 	gint             prop_id = 0;
+	gboolean change = FALSE;
 
 	g_return_if_fail (subject != NULL);
 	g_return_if_fail (predicate != NULL);
@@ -1640,19 +1709,23 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 			             "Class '%s' not found in the ontology", object);
 			return;
 		}
+
+		change = TRUE;
 	} else if (strcmp (predicate, TRACKER_PREFIX "uri") == 0) {
 		/* internal property tracker:uri, used to change uri of existing element */
 		resource_buffer->new_subject = g_strdup (object);
+
+		change = TRUE;
 	} else {
 		/* add value to metadata database */
-		cache_set_metadata_decomposed (property, object, graph, &actual_error);
+		change = cache_set_metadata_decomposed (property, object, graph, &actual_error);
 		if (actual_error) {
 			tracker_data_update_buffer_clear ();
 			g_propagate_error (error, actual_error);
 			return;
 		}
 
-		if (insert_callbacks) {
+		if (insert_callbacks && change) {
 			guint n;
 			for (n = 0; n < insert_callbacks->len; n++) {
 				TrackerStatementDelegate *delegate;
@@ -1665,7 +1738,7 @@ tracker_data_insert_statement_with_uri (const gchar            *graph,
 		}
 	}
 
-	if (!in_journal_replay) {
+	if (!in_journal_replay && change) {
 		tracker_db_journal_append_insert_statement_id (
 			(graph != NULL ? query_resource_id (graph) : 0),
 			resource_buffer->id,
@@ -1684,6 +1757,8 @@ tracker_data_insert_statement_with_string (const gchar            *graph,
 	GError          *actual_error = NULL;
 	TrackerProperty *property;
 	gint             id = 0;
+	gboolean change;
+
 
 	g_return_if_fail (subject != NULL);
 	g_return_if_fail (predicate != NULL);
@@ -1714,14 +1789,14 @@ tracker_data_insert_statement_with_string (const gchar            *graph,
 	}
 
 	/* add value to metadata database */
-	cache_set_metadata_decomposed (property, object, graph, &actual_error);
+	change = cache_set_metadata_decomposed (property, object, graph, &actual_error);
 	if (actual_error) {
 		tracker_data_update_buffer_clear ();
 		g_propagate_error (error, actual_error);
 		return;
 	}
 
-	if (insert_callbacks) {
+	if (insert_callbacks && change) {
 		guint n;
 		for (n = 0; n < insert_callbacks->len; n++) {
 			TrackerStatementDelegate *delegate;
@@ -1733,7 +1808,7 @@ tracker_data_insert_statement_with_string (const gchar            *graph,
 		}
 	}
 
-	if (!in_journal_replay) {
+	if (!in_journal_replay && change) {
 		tracker_db_journal_append_insert_statement (
 			(graph != NULL ? query_resource_id (graph) : 0),
 			resource_buffer->id,
@@ -2005,6 +2080,12 @@ tracker_data_commit_transaction (void)
 		update_buffer.fts_ever_updated = FALSE;
 	}
 
+	if (update_buffer.class_counts) {
+		/* successful transaction, no need to rollback class counts,
+		   so remove them */
+		g_hash_table_remove_all (update_buffer.class_counts);
+	}
+
 	iface = tracker_db_manager_get_db_interface ();
 
 	tracker_db_interface_end_transaction (iface);
@@ -2246,56 +2327,10 @@ tracker_data_delete_resource_description (const gchar *graph,
 }
 
 
-void
-tracker_data_update_sparql (const gchar  *update,
-                            GError      **error)
-{
-	GError *actual_error = NULL;
-	TrackerDBInterface *iface;
-	TrackerSparqlQuery *sparql_query;
-
-	g_return_if_fail (update != NULL);
-
-	iface = tracker_db_manager_get_db_interface ();
-
-	sparql_query = tracker_sparql_query_new_update (update);
-	resource_time = time (NULL);
-	tracker_db_interface_execute_query (iface, NULL, "SAVEPOINT sparql");
-	tracker_db_journal_start_transaction (resource_time);
-
-	tracker_sparql_query_execute_update (sparql_query, FALSE, &actual_error);
-
-	if (actual_error) {
-		tracker_data_update_buffer_clear ();
-		tracker_db_interface_execute_query (iface, NULL, "ROLLBACK TO sparql");
-		tracker_db_journal_rollback_transaction ();
-
-		if (rollback_callbacks) {
-			guint n;
-			for (n = 0; n < rollback_callbacks->len; n++) {
-				TrackerCommitDelegate *delegate;
-				delegate = g_ptr_array_index (rollback_callbacks, n);
-				delegate->callback (delegate->user_data);
-			}
-		}
-
-		g_propagate_error (error, actual_error);
-
-		g_object_unref (sparql_query);
-		return;
-	}
-
-	tracker_db_journal_commit_transaction ();
-	resource_time = 0;
-	tracker_db_interface_execute_query (iface, NULL, "RELEASE sparql");
-
-	g_object_unref (sparql_query);
-}
-
-
-GPtrArray *
-tracker_data_update_sparql_blank (const gchar  *update,
-                                  GError      **error)
+static GPtrArray *
+update_sparql (const gchar  *update,
+               gboolean      blank,
+               GError      **error)
 {
 	GError *actual_error = NULL;
 	TrackerDBInterface *iface;
@@ -2304,6 +2339,12 @@ tracker_data_update_sparql_blank (const gchar  *update,
 
 	g_return_val_if_fail (update != NULL, NULL);
 
+	if (!tracker_db_manager_has_enough_space ()) {
+		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_NO_SPACE,
+			"There is not enough space on the file system for update operations");
+		return NULL;
+	}
+
 	iface = tracker_db_manager_get_db_interface ();
 
 	sparql_query = tracker_sparql_query_new_update (update);
@@ -2312,7 +2353,7 @@ tracker_data_update_sparql_blank (const gchar  *update,
 	tracker_db_interface_execute_query (iface, NULL, "SAVEPOINT sparql");
 	tracker_db_journal_start_transaction (resource_time);
 
-	blank_nodes = tracker_sparql_query_execute_update (sparql_query, TRUE, &actual_error);
+	blank_nodes = tracker_sparql_query_execute_update (sparql_query, blank, &actual_error);
 
 	if (actual_error) {
 		tracker_data_update_buffer_clear ();
@@ -2338,11 +2379,31 @@ tracker_data_update_sparql_blank (const gchar  *update,
 	resource_time = 0;
 	tracker_db_interface_execute_query (iface, NULL, "RELEASE sparql");
 
+	if (update_buffer.class_counts) {
+		/* successful transaction, no need to rollback class counts,
+		   so remove them */
+		g_hash_table_remove_all (update_buffer.class_counts);
+	}
+
 	g_object_unref (sparql_query);
 
 	return blank_nodes;
 }
 
+void
+tracker_data_update_sparql (const gchar  *update,
+                            GError      **error)
+{
+	update_sparql (update, FALSE, error);
+}
+
+GPtrArray *
+tracker_data_update_sparql_blank (const gchar  *update,
+                                  GError      **error)
+{
+	return update_sparql (update, TRUE, error);
+}
+
 void
 tracker_data_sync (void)
 {
