@@ -24,7 +24,7 @@ G_BEGIN_DECLS
 
 int tracker_fts_init (sqlite3 *db);
 int tracker_fts_update_init (int id);
-int tracker_fts_update_text (int id, int column_id, const char *text);
+int tracker_fts_update_text (int id, int column_id, const char *text, gboolean limit_word_length);
 void tracker_fts_update_commit (void);
 void tracker_fts_update_rollback (void);
 gchar * tracker_fts_get_create_fts_table_query (void);
