@@ -24,21 +24,11 @@
 #include <glib.h>
 
 #include <libtracker-common/tracker-os-dependant.h>
-#include <libtracker-common/tracker-ontologies.h>
-#include <libtracker-common/tracker-utils.h>
 
 #include <libtracker-extract/tracker-extract.h>
-#include <libtracker-extract/tracker-utils.h>
 
 #include "tracker-main.h"
 
-#define NIE_PREFIX TRACKER_NIE_PREFIX
-#define NFO_PREFIX TRACKER_NFO_PREFIX
-#define NCO_PREFIX TRACKER_NCO_PREFIX
-
-#define RDF_PREFIX TRACKER_RDF_PREFIX
-#define RDF_TYPE RDF_PREFIX "type"
-
 typedef enum {
 	READ_TITLE,
 	READ_SUBJECT,
@@ -97,7 +87,7 @@ extract_content (const gchar *path,
 		return NULL;
 	}
 
-	text = tracker_extract_text_normalize (output, n_words, NULL);
+	text = tracker_text_normalize (output, n_words, NULL);
 
 	g_free (command);
 	g_free (output);
@@ -284,7 +274,7 @@ text_handler (GMarkupParseContext  *context,
 		tracker_sparql_builder_object_unvalidated (metadata, text);
 		break;
 	case READ_CREATED:
-		date = tracker_extract_guess_date (text);
+		date = tracker_date_guess (text);
 		tracker_sparql_builder_predicate (metadata, "nie:contentCreated");
 		tracker_sparql_builder_object_unvalidated (metadata, date);
 		g_free (date);
