@@ -333,9 +333,9 @@ miner_files_constructed (GObject *object)
 	mf = TRACKER_MINER_FILES (object);
 	fs = TRACKER_MINER_FS (object);
 
-	if (!mf->private->config) {
-		g_critical ("No config. This is mandatory");
-		g_assert_not_reached ();
+	if (G_UNLIKELY (!mf->private->config)) {
+		g_message ("No config for miner %p (%s).", object, G_OBJECT_TYPE_NAME (object));
+		return;
 	}
 
 	if (tracker_config_get_index_removable_devices (mf->private->config)) {
@@ -1184,6 +1184,10 @@ miner_files_check_file (TrackerMinerFS *fs,
 	/* Check module file ignore patterns */
 	mf = TRACKER_MINER_FILES (fs);
 
+	if (G_UNLIKELY (!mf->private->config)) {
+		return TRUE;
+	}
+
 	return tracker_miner_files_check_file (file,
 	                                       tracker_config_get_ignored_file_paths (mf->private->config),
 	                                       tracker_config_get_ignored_file_patterns (mf->private->config));
@@ -1198,6 +1202,10 @@ miner_files_check_directory (TrackerMinerFS *fs,
 	/* Check module file ignore patterns */
 	mf = TRACKER_MINER_FILES (fs);
 
+	if (G_UNLIKELY (!mf->private->config)) {
+		return TRUE;
+	}
+
 	return tracker_miner_files_check_directory (file,
 	                                            tracker_config_get_index_recursive_directories (mf->private->config),
 	                                            tracker_config_get_index_single_directories (mf->private->config),
@@ -1216,6 +1224,10 @@ miner_files_check_directory_contents (TrackerMinerFS *fs,
 
 	mf = TRACKER_MINER_FILES (fs);
 
+	if (G_UNLIKELY (!mf->private->config)) {
+		return TRUE;
+	}
+
 	return tracker_miner_files_check_directory_contents (parent,
 	                                                     children,
 	                                                     tracker_config_get_ignored_directories_with_content (mf->private->config));
@@ -1229,6 +1241,10 @@ miner_files_monitor_directory (TrackerMinerFS *fs,
 
 	mf = TRACKER_MINER_FILES (fs);
 
+	if (G_UNLIKELY (!mf->private->config)) {
+		return TRUE;
+	}
+
 	return tracker_miner_files_monitor_directory (file,
 	                                              tracker_config_get_enable_monitors (mf->private->config),
 	                                              mf->private->index_single_directories);
@@ -1865,30 +1881,13 @@ tracker_miner_files_monitor_directory (GFile    *file,
                                        gboolean  enable_monitors,
                                        GSList   *directories_to_check)
 {
-	GSList *l;
-
 	if (!enable_monitors) {
 		return FALSE;
 	}
 
-	/* We don't want child directories inside IndexSingleDirectories
-	 * to have a monitor added.
-	 */
-	for (l = directories_to_check; l; l = l->next) {
-		gboolean is_child = FALSE;
-		GFile *dir;
-
-		dir = g_file_new_for_path (l->data);
-		is_child = g_file_has_prefix (file, dir);
-		g_object_unref (dir);
-
-		if (is_child) {
-			return FALSE;
-		}
-	}
-
-	/* Fallback to the check directory routine, since we don't
-	 * monitor anything we don't process.
-	 */
+	/* We'll only get this signal for the directories where check_directory()
+	 * and check_directory_contents() returned TRUE, so by default we want
+	 * these directories to be indexed.
+         */
 	return TRUE;
 }
